/*
 * options.c - handles option processing for PPP.
 *
 * Copyright (c) 1989 Carnegie Mellon University.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms are permitted
 * provided that the above copyright notice and this paragraph are
 * duplicated in all such forms and that any documentation,
 * advertising materials, and other materials related to such
 * distribution and use acknowledge that the software was developed
 * by Carnegie Mellon University.  The name of the
 * University may not be used to endorse or promote products derived
 * from this software without specific prior written permission.
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
 * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 */

#define RCSID	"$Id: options.c,v 1.6 2002/03/21 00:25:06 mhuang Exp $"

#include <getopt.h>
#include <stdlib.h>
#include <string.h>
#ifndef NO_SYSLOG // by XXXXXXXX (2007/02/27)
#include <syslog.h>
#endif // NO_SYSLOG }
#include "pppd.h"
#include "fsm.h"
#include "lcp.h"
#include "ipcp.h"

int	debug = 0;		/* Debug flag */
int	kdebugflag = 0;		/* Tell kernel to print debug messages */
int	default_device = 1;	/* Using /dev/tty or equivalent */
char	devnam[MAXPATHLEN];	/* Device name */
bool	nodetach = 0;		/* Don't detach from controlling tty */
bool	updetach = 0;		/* Detach once link is up */
int	maxconnect = 0;		/* Maximum connect time */
char	user[MAXNAMELEN];	/* Username for PAP */
char	passwd[MAXSECRETLEN];	/* Password for PAP */
bool	persist = 0;		/* Reopen link after it goes down */
char	our_name[MAXNAMELEN];	/* Our name for authentication purposes */
bool	demand = 0;		/* do dial-on-demand */
char	*ipparam = NULL;	/* Extra parameter for ip up/down scripts */
int	idle_time_limit = 0;	/* Disconnect if idle for this many seconds */
int	holdoff = 30;		/* # seconds to pause before reconnecting */
bool	holdoff_specified;	/* true if a holdoff value has been given */
int	log_to_fd = 1;		/* send log messages to this fd too */
bool	log_default = 1;	/* log_to_fd is default (stdout) */
int	maxfail = 10;		/* max # of unsuccessful connection attempts */
char	linkname[MAXPATHLEN];	/* logical name for link */
bool	tune_kernel = 1;	/* may alter kernel settings */
int	connect_delay = 1000;	/* wait this many ms after connect script */
int	req_unit = -1;		/* requested interface unit */
char	*bundle_name = NULL;	/* bundle name for multilink */
bool	dump_options;		/* print out option values */
bool	dryrun;			/* print out option values and exit */
char	*domain;		/* domain name set by domain option */
int	baud_rate;		/* Actual bits/second for serial device */

char *current_option;		/* the name of the option being parsed */
int  privileged_option;		/* set iff the current option came from root */
char *option_source;		/* string saying where the option came from */
int  option_priority = OPRIO_CFGFILE; /* priority of the current options */
bool devnam_fixed;		/* can no longer change device name */

char service_name[255];
extern char *pppoe_srv_name;

extern int setdevname_pppoe(const char *cp);

// from ./auth.c
extern bool refuse_pap;		/* Don't wanna auth. ourselves with PAP */
extern bool refuse_chap;		/* Don't wanna auth. ourselves with CHAP */

// modify for WHR2-G54V v2.03 2003.11.15
bool   continuous = 0;		//³
bool   force_connect = 0;
bool   act_txonly = 0;
bool   act_rxonly = 0;
extern bool   force_resolv;
// done.. for WHR2-G54V v2.03 2003.11.15

#if 1
/*
 * Valid arguments.
 */
option_t general_options[] = {
/*    { "debug", o_int, &debug,
      "Increase debugging level", OPT_INC | OPT_NOARG | 1 },
    { "-d", o_int, &debug,
      "Increase debugging level",
      OPT_ALIAS | OPT_INC | OPT_NOARG | 1 },
*/
    { "kdebug", o_int, &kdebugflag,
      "Set kernel driver debug level", OPT_PRIO },

    { "nodetach", o_bool, &nodetach,
      "Don't detach from controlling tty", OPT_PRIO | 1 },
    { "-detach", o_bool, &nodetach,
      "Don't detach from controlling tty", OPT_ALIAS | OPT_PRIOSUB | 1 },
    { "updetach", o_bool, &updetach,
      "Detach from controlling tty once link is up",
      OPT_PRIOSUB | OPT_A2CLR | 1, &nodetach },

    { "holdoff", o_int, &holdoff,
      "Set time in seconds before retrying connection", OPT_PRIO },

    { "idle", o_int, &idle_time_limit,
      "Set time in seconds before disconnecting idle link", OPT_PRIO },

    { "maxconnect", o_int, &maxconnect,
      "Set connection time limit",
      OPT_PRIO | OPT_LLIMIT | OPT_NOINCR | OPT_ZEROINF },

/*    { "domain", o_special, (void *)setdomain,
      "Add given domain name to hostname",
      OPT_PRIO | OPT_PRIV | OPT_A2STRVAL, &domain },

    { "file", o_special, (void *)readfile,
      "Take options from a file", OPT_NOPRINT },
    { "call", o_special, (void *)callfile,
      "Take options from a privileged file", OPT_NOPRINT },
*/
    { "persist", o_bool, &persist,
      "Keep on reopening connection after close", OPT_PRIO | 1 },
    { "nopersist", o_bool, &persist,
      "Turn off persist option", OPT_PRIOSUB },

    { "demand", o_bool, &demand,
      "Dial on demand", OPT_INITONLY | 1, &persist },

/*    { "--version", o_special_noarg, (void *)showversion,
      "Show version number" },
    { "--help", o_special_noarg, (void *)showhelp,
      "Show brief listing of options" },
    { "-h", o_special_noarg, (void *)showhelp,
      "Show brief listing of options", OPT_ALIAS },

    { "logfile", o_special, (void *)setlogfile,
      "Append log messages to this file",
      OPT_PRIO | OPT_A2STRVAL | OPT_STATIC, &logfile_name },
    { "logfd", o_int, &log_to_fd,
      "Send log messages to this file descriptor",
      OPT_PRIOSUB | OPT_A2CLR, &log_default },
    { "nolog", o_int, &log_to_fd,
      "Don't send log messages to any file",
      OPT_PRIOSUB | OPT_NOARG | OPT_VAL(-1) },
    { "nologfd", o_int, &log_to_fd,
      "Don't send log messages to any file descriptor",
      OPT_PRIOSUB | OPT_ALIAS | OPT_NOARG | OPT_VAL(-1) },
*/
    { "linkname", o_string, linkname,
      "Set logical name for link",
      OPT_PRIO | OPT_PRIV | OPT_STATIC, NULL, MAXPATHLEN },

    { "maxfail", o_int, &maxfail,
      "Maximum number of unsuccessful connection attempts to allow",
      OPT_PRIO },

    { "ktune", o_bool, &tune_kernel,
      "Alter kernel settings as necessary", OPT_PRIO | 1 },
    { "noktune", o_bool, &tune_kernel,
      "Don't alter kernel settings", OPT_PRIOSUB },

    { "connect-delay", o_int, &connect_delay,
      "Maximum time (in ms) to wait after connect script finishes",
      OPT_PRIO },

    { "unit", o_int, &req_unit,
      "PPP interface unit number to use if possible",
      OPT_PRIO | OPT_LLIMIT, 0, 0 },

/*    { "dump", o_bool, &dump_options,
      "Print out option values after parsing all options", 1 },
    { "dryrun", o_bool, &dryrun,
      "Stop after parsing, printing, and checking options", 1 },
*/
#ifdef HAVE_MULTILINK
    { "multilink", o_bool, &multilink,
      "Enable multilink operation", OPT_PRIO | 1 },
    { "mp", o_bool, &multilink,
      "Enable multilink operation", OPT_PRIOSUB | OPT_ALIAS | 1 },
    { "nomultilink", o_bool, &multilink,
      "Disable multilink operation", OPT_PRIOSUB | 0 },
    { "nomp", o_bool, &multilink,
      "Disable multilink operation", OPT_PRIOSUB | OPT_ALIAS | 0 },

    { "bundle", o_string, &bundle_name,
      "Bundle name for multilink", OPT_PRIO },
#endif /* HAVE_MULTILINK */

#ifdef PLUGIN
    { "plugin", o_special, (void *)loadplugin,
      "Load a plug-in module into pppd", OPT_PRIV | OPT_A2LIST },
#endif

#ifdef PPP_FILTER
    { "pdebug", o_int, &dflag,
      "libpcap debugging", OPT_PRIO },

    { "pass-filter", 1, setpassfilter,
      "set filter for packets to pass", OPT_PRIO },

    { "active-filter", 1, setactivefilter,
      "set filter for active pkts", OPT_PRIO },
#endif

// modify for WHR2-G54V v2.03 2003.11.15
    { "continuous", o_bool, &continuous,
      "Continuous", OPT_PRIO|1 },
    { "active-txonly", o_bool, &act_txonly,
      "set trancemit only for active pkts", OPT_PRIO|1 },
    { "active-rxonly", o_bool, &act_rxonly,
      "set recive only for active pkts",    OPT_PRIO|1 },
    { "force-resolv", o_bool, &force_resolv,
      "force write resolv.conf", OPT_PRIO|1 },
    { "force"     , o_bool, &force_connect,
      "Force connect at once", OPT_PRIO|1 },
// done.. for WHR2-G54V v2.03 2003.11.15

    { NULL }
};

#endif

//static char *usage_string = "usage: %s interface -d -k [-i idle] [-u username] [-p passwd] [-r mru] [-t mtu]\n";

// 2002.9 hirai
//add options -c 
static char *usage_string = "usage: %s interface -d -k -b "
                            "[-i idle] [-u username] [-p passwd] [-r mru] [-t mtu] [-c config_file]"
#ifdef CONFIG_WAN_IPUNNUMBERD     // Appended by XXXXXXXXXXXX 2007/03/14
                            " [-x]"
#endif  //CONFIG_WAN_IPUNNUMBERD  // Appended by XXXXXXXXXXXX 2007/03/14
                            " [-n unit] \n";

char pppoecd_configfile[255]={0};

#define FUNC_TRACE(x)   /*printf x*/

//----------------------------------------------------------------------
void disp_lcp_opt(lcp_options *lcp, char *msg)
{
	cprintf( "--- LCP %s ---\n",msg);
	cprintf( "neg_mru=%d neg_upap=%d neg_chap=%d silent=%d neg_cbcp=%d mru=%d\n"
		,lcp->neg_mru
		,lcp->neg_upap
		,lcp->neg_chap
		,lcp->silent
		,lcp->neg_cbcp
		,lcp->mru
		);
		
}
//----------------------------------------------------------------------
void disp_ipcp_opt(ipcp_options *lcp, char *msg)
{
	cprintf( "--- IPCP %s ---\n",msg);
	cprintf( "neg_addr=%d old_addrs=%d req_addr=%d default_route=%d proxy_arp=%d neg_vj=%d\n"
		,lcp->neg_addr
		,lcp->old_addrs
		,lcp->req_addr
		,lcp->default_route
		,lcp->proxy_arp
		,lcp->neg_vj
		);
	cprintf( "old_vj=%d accept_local=%d accept_remote=%d req_dns1=%d req_dns2=%d vj_protocol=%d\n"
		,lcp->old_vj
		,lcp->accept_local
		,lcp->accept_remote
		,lcp->req_dns1
		,lcp->req_dns2
		,lcp->vj_protocol
		);
		
}
//----------------------------------------------------------------------
void
disp_all_optionval(void)
{
	// from pppoe.c
	extern bool	pppoe_server;		//PPPoE listen for incoming requests
	extern char	*pppoe_srv_name;
	extern char	*pppoe_ac_name;
	extern char    *hostuniq;
	extern int     retries;
	///
	
/*
	disp_lcp_opt(&lcp_allowoptions[0], "ALLOW");
	disp_lcp_opt(&lcp_wantoptions[0], "WANT");
	
	disp_ipcp_opt(&ipcp_allowoptions[0], "ALLOW");
	disp_ipcp_opt(&ipcp_wantoptions[0], "WANT");
*/	
	cprintf( "--- PPPoE ---\n");
	cprintf( "pppoe_srv_name=%s pppoe_ac_name=%s hostuniq=%s\nretries=%d pppoe_server=%d ifunit=%d\n"
		,pppoe_srv_name, pppoe_ac_name, hostuniq
		,retries, pppoe_server,ifunit);
		
	cprintf( "--- Auth ---\n");
	cprintf( "user=(%s)  pass=(%s)  refuse_chap=%d refuse_pap=%d\n",user,passwd, refuse_chap, refuse_pap);
	
	cprintf( "--- General Opt ---\n");
	cprintf( "holdoff=%d idle_time_limit=%d maxconnect=%d persist=%d\ndemand=%d maxfail=%d connect_delay=%d continuous=%d\n\n"
		,holdoff, idle_time_limit, maxconnect, persist
		,demand, maxfail, connect_delay, continuous);
}

//----------------------------------------------------------------------
//ޥɥץ롼 
//----------------------------------------------------------------------
//----------------------------------------------------------------------
//ƬΥڡϥåפ 
char *strip_spc(char *msg)
{
	if (msg==NULL || *msg==0){
		return msg;
	}

	//ƬΥڡϽ
	while (*msg==' '){
		msg ++;
	}
	return msg;
}
//----------------------------------------------------------------------
//ƬΥڡϼĤˤ륹ڡNULL,ԡ֤ߡdstNULLü롣
char *stripmsg(char *dst, char *msg, int dstlen)
{
	if (msg==NULL || *msg==0 || dstlen<=0){
		return msg;
	}

	//ƬΥڡϽ
	while (*msg==' '){
		msg ++;
	}
	dstlen--;  // NULLüʬ
	//NULL,ԡڡ
	while (*msg!=0 && *msg!='\r' && *msg!='\n' && *msg!=' ' && *msg!='\t' && dstlen>0){
		*dst++ = *msg++;
		dstlen--;
	}
	*dst = 0;  // NULLü
	return msg;	//ΰ֤֤
}

//----------------------------------------------------------------------
/*
 * number_option - parse an unsigned numeric parameter for an option.
 */
static int
number_option(char *str, u_int32_t *valp, int base)
{
    char *ptr;

    *valp = strtoul(str, &ptr, base);
    if (ptr == str) {
	option_error("invalid numeric parameter '%s' for %s option",
		     str, current_option);
	return 0;
    }
    return 1;
}


//----------------------------------------------------------------------
// pppd/options.c process_option()¸
// Υ⥸塼 link ǤʤἫ
// ñư¸ʤ addr2 ƤΤϡա
/*
	Ȥ LCP noaccompϲĤĴʤФʤʤ
	lcp_wantoptions[0].neg_accompression
	lcp_allowoptions[0].neg_accompression
*/
static int
process_option(option_t *opt, char *arg)
{
    u_int32_t v;
    int iv, a;
	int retval=-1;
	
	FUNC_TRACE(("process_option(%s=%s)\n",opt->name,arg));
	
	switch (opt->type) {
    case o_bool:
		FUNC_TRACE(("o_bool NAME=%s\n", opt->name));
		v = opt->flags & OPT_VALUE;
		*(bool *)(opt->addr) = v;
		if (opt->addr2 && (opt->flags & OPT_A2COPY))
		    *(bool *)(opt->addr2) = v;
	    retval = 0;
		break;
		
    case o_int:
		FUNC_TRACE(("o_int NAME=%s\n", opt->name));
		iv = atoi(arg);
		//ʥץ󤬤ꡢۤȤɤ̤ݡȤǤ
		//Ǥͤ
		*(int *)(opt->addr) = iv;
		if (opt->addr2 && (opt->flags & OPT_A2COPY))
		    *(int *)(opt->addr2) = iv;
	    else
	    if (opt->addr2)
			*(bool *)(opt->addr2) = 1;
	    
	    retval = 0;
		break;
		
    case o_uint32:
		if (opt->flags & OPT_NOARG) {
		    v = opt->flags & OPT_VALUE;
		    if (v & 0x80)
			    v |= 0xffffff00U;
		} else if (!number_option(arg, &v, 16)){
			//return 0;
			break;
		}
		if (opt->flags & OPT_OR)
		    v |= *(u_int32_t *)(opt->addr);
		*(u_int32_t *)(opt->addr) = v;
		if (opt->addr2 && (opt->flags & OPT_A2COPY))
		    *(u_int32_t *)(opt->addr2) = v;
	    retval = 0;
		break;

    case o_string:
		FUNC_TRACE(("o_string NAME=%s\n", opt->name));
		if (opt->flags & OPT_STATIC) {
		    strlcpy((char *)(opt->addr), arg, opt->upper_limit);
		} else {
			char *sv;
		    sv = strdup(arg);
		    if (sv == NULL){
//				novm("option argument");
				ERR_PRINTF("strdup fail\n");
				break;
			}
		    *(char **)(opt->addr) = sv;
		}
	    retval = 0;
		break;
	default :
		ERR_PRINTF("unsport name\n");
	}
	if (retval){
#ifndef NO_SYSLOG // by XXXXXXXX (2007/02/27)
		syslog(LOG_ERR, "pppd command fail (%s)",opt->name);
		ERR_PRINTF("NAME=%s\n", opt->name);
#endif //NO_SYSLOG }
	}else{
	    if (opt->addr2 && (opt->flags & (OPT_A2COPY|OPT_ENABLE
			|OPT_A2PRINTER|OPT_A2STRVAL|OPT_A2LIST)) == 0)
		*(bool *)(opt->addr2) = !(opt->flags & OPT_A2CLR);
	}
	return retval;
}

//----------------------------------------------------------------------
/*
 * match_option - see if this option matches an option_t structure.
 */
static int
match_option(name, opt, dowild)
    char *name;
    option_t *opt;
    int dowild;
{
	int (*match) __P((char *, char **, int));
	
	FUNC_TRACE((">match_option (%s)\n",name));
	
	if (dowild != (opt->type == o_wild))
		return 0;
	if (!dowild)
		return strcmp(name, opt->name) == 0;
	match = (int (*) __P((char *, char **, int))) opt->addr;
	return (*match)(name, NULL, 0);
}

//----------------------------------------------------------------------
//name 򥪥ץˤĤΤõ
static option_t *
find_option(const char *name)
{
	option_t *opt;
	int i, dowild;
	extern option_t pppoe_options[];		// from ./pppoe.c
	extern option_t general_options[];		// from ./options.c
	extern struct protent *protocols[];		// from main.c
	
	for (dowild = 0; dowild <= 1; ++dowild) {
		for (i = 0; protocols[i] != NULL; ++i)
			if ((opt = protocols[i]->options) != NULL)
				for (; opt->name != NULL; ++opt)
					if (match_option(name, opt, dowild)){
						FUNC_TRACE((">found opt %s [%s]\n",name,opt->description));
						return opt;
					}
	}
	
	// search pppoe.c
	for (opt = pppoe_options; opt->name != NULL; ++opt)
		if (match_option(name, opt, 0)){
			FUNC_TRACE((">found opt %s [%s]\n",name,opt->description));
			return opt;
		}
	
	// search options.c
	for (opt = general_options; opt->name != NULL; ++opt)
		if (match_option(name, opt, 0)){
			FUNC_TRACE((">found opt %s [%s]\n",name,opt->description));
			return opt;
		}
	
#ifndef NO_SYSLOG // by XXXXXXXX (2007/02/27)
	syslog(LOG_ERR, "pppd (%s) command fail (%s)",__FUNCTION__, name);
	ERR_PRINTF("not found (%s)\n", name);
#endif //NO_SYSLOG }
	return NULL;
}
//----------------------------------------------------------------------
int parse_configfile()
{
	FILE *fp;
	int retval=-1;
	
	//printf(">parse_configfile (%s)\n",pppoecd_configfile);
	
	if (pppoecd_configfile[0]==0)
		return 0;
		
	if ((fp=fopen(pppoecd_configfile, "r"))!=NULL){
	//	printf("pppoecd : ****** open config file (%s) ********\n",pppoecd_configfile);
		while (!feof(fp)){
			char line[255];
			line[0] = 0;
			
			if (fgets(line, sizeof(line), fp)!=NULL){
				char *p;
				char cmdopt[128], cmdval[128];
				option_t *tmp_optt;
				
			//	printf("(%s)",line);
				
				if (line[0]=='#')   continue;      // comment line
				
				p = strip_spc(line);
				p = stripmsg(cmdopt, p, sizeof(cmdopt));
				p = stripmsg(cmdval, p, sizeof(cmdval));
				
				if (cmdopt[0]==0)
					continue;
				
				retval = -1;
			//	printf("-- %s  %s\n",cmdopt,cmdval);
				if ((tmp_optt=find_option(cmdopt))){
			//		printf("name  = %s [%s]\n",tmp_optt->name, tmp_optt->description);
					if (process_option(tmp_optt, cmdval)==0)
						retval = 0;
					else{
						break;
					}
				}else{
					break;
				}
			}else
				break;
		}
		fclose(fp);
	}else{
		ERR_PRINTF("configfile (%s) open error\n",pppoecd_configfile);
	}
	return retval;
}

/*
 * parse_args - parse a string of arguments from the command line.
 */
int
parse_args(argc, argv)
    int argc;
    char **argv;
{
    int opt;
	
	
    while ((opt = getopt(argc, argv, "dkbxi:u:p:r:t:a:s:c:n:")) != -1) {
	    switch (opt) {
	    case 'd':
		    //debug = nodetach = 1;
		    debug = 1;
		    break;
	    case 'k':
		    persist = 1;
		    break;
#ifdef CONFIG_WAN_IPUNNUMBERD     // Appended by XXXXXXXXXXXX 2007/03/14
	    case 'x':
		    ipcp_wantoptions[0].unnumbered = 1;
		    break;
#endif  //CONFIG_WAN_IPUNNUMBERD  // Appended by XXXXXXXXXXXX 2007/03/14
	    case 'i':
		    idle_time_limit = atoi(optarg);
/*
		    if (idle_time_limit > 0)
			    demand = 1;
*/
		    break;
	    case 'b':
			continuous = 1;
		    break;
	    case 'u':
		    strncpy(user, optarg, MAXNAMELEN);
		    strncpy(our_name, optarg, MAXNAMELEN);
		    break;
	    case 'p':
		    strncpy(passwd, optarg, MAXSECRETLEN);
		    break;
/*	    case 'r':
		    lcp_wantoptions[0].neg_mru = 1;
		    lcp_wantoptions[0].mru = atoi(optarg);
		    break;
	    case 't':
		    lcp_allowoptions[0].mru = atoi(optarg);
		    break;
*/
		case 'a':
			switch (atoi(optarg))
			{
				case 0: /* auto */
//puts("auth auto"); //Commented for XXXXXXXX by XXXXXXXXXXXX (2004/10/09)
					lcp_wantoptions[0].neg_upap = 1;
					lcp_wantoptions[0].neg_chap = 1;
					break;

				case 1: /* pap */
//puts("auth pap"); //Commented for XXXXXXXX by XXXXXXXXXXXX (2004/10/09)
					lcp_wantoptions[0].neg_upap = 1;
					lcp_wantoptions[0].neg_chap = 0;
					refuse_chap = 1;		//ݤ 
					break;

				case 2: /* chap */
//puts("auth chap"); //Commented for XXXXXXXX by XXXXXXXXXXXX (2004/10/09)
					lcp_wantoptions[0].neg_upap = 0;
					lcp_wantoptions[0].neg_chap = 1;
					refuse_pap = 1;			//ݤ 
					break;
			}
			break;
/*		case 's':
		    strncpy(service_name, optarg, 255);
			if (strcmp(service_name, ""))
				pppoe_srv_name = service_name;
			break;
*/			
		case 'c':
			strncpy(pppoecd_configfile, optarg, sizeof(pppoecd_configfile));
			//printf("config file=(%s)\n",pppoecd_configfile);
			break;
	    case 'n':
		    ifunit   = atoi(optarg);
		    req_unit = ifunit;
			break;
	    default:
		    fprintf(stderr, usage_string, argv[0]);
		    return 0;
	    }
    }

	if (parse_configfile()){
#ifndef NO_SYSLOG // by XXXXXXXX (2007/02/27)
	    syslog(LOG_ERR, "pppd exit by error [config file error]");
#endif //NO_SYSLOG }
		//printf("pppoecd: ** exit by error\n");
		exit(EXIT_OPTION_ERROR);
	}
	
	disp_all_optionval();
/////

    if (optind < argc)
	    setdevname_pppoe(argv[optind]);
    else {
	    fprintf(stderr, usage_string, argv[0]);
	    return 0;
    }

    return 1;
}
